# Purpose: Simulate case-control data and case-parent (i.e. trio) data that use an overlapping set 
#          of cases, to examine the power of: 
#
#          1.  Zmix : the combination of Zpop & Zfam w/ Chen & Lin weights 
#                     (i.e. weigths that are based on sd and cov)
#          2.  Zpop : the trend test
#          3.  Zfam : the tdt
#          4.  Zmax : Zmix with the theoretically optimal weights that maximize power
#
# under different alternarives. 
# 
# The parameters for generating the data are given in 'popfam-sim.par'.
#
# Note that : 
#
#      - although the Chen & Lin weights are pretty good, they are not optimal. further, 
#        since the opt weight varies with the marginal power of Zpop and Zfam, which in 
#        turn depends upon the penetrance, the sample size (which does not contain 
#        information about penetrance) is not sufficient for determining the optimal weight.
#        Moreover, weights that are based exclusively on sample size, are likely to perform 
#        worse than Chen & Lin weigths, since the latter accounts for the covariance btwn Zpop 
#        and Zfam. 
#      - it appears that the type 1 error cannot be controlled if one estimates the optimal 
#        weight from the available data, although a permutation-based strategy may be 
#        possible. Furthermore, due to the winner's curse and publication bias, estimates 
#        form independent data set(s) are also unlikely to control the type 1 error.
#      - this code assumes that the SNP and the DL (disease locus) are completely linked (i.e. theta = 0)
#      - one should set diseq = 0 in 'popfam-sim.par' to simulate under the null. 
#      - hij means 'the haplotype with allele 'i' at the SNP and 'j' at the DL 
#      - diseq pertains to h11 - daf*maf and it is assumed to be greater than or equal to zero
#      - this code assumes diallelic DL, diallelic SNP, and HWE
#      - At least in terms of the simulated case-control and case-parent data, this code agrees 
#        with Chen & Lin (2008) 

# This function returns the unconditional covariance btwn 
# Zpop and Zfam.
uccov <- function(maf, ncase, ncont)
{
  het   <- 2*maf*(1-maf)
  nhp   <- het * 2 * ncase

  # Note: the Trend test compares the difference in expected dose. So,
  # genos are coded as 0, 1, and 2, and when there is random mating,
  # each geno ~ Bin(2,maf) w/ mean 2*maf, and variance 2*maf*(1-maf).
  # Hence, the variance of the mean geno (which estimates the
  # expected dose) is 2*maf*(1-maf)/N, where N is ncase for cases,
  # and ncont for controls. Under HWE, the Trend test is the same
  # as the test that compares allele freqs btwn cases and controls.

  sigma <- sqrt(2 * maf*(1-maf)*( 1/ncase + 1/ncont ) )

  # When HWE does not hold, the distribution of each geno is no
  # longer Bin(2,maf), because the component alleles are no longer
  # independent Ber(maf) rv's. Needless to say, the variance of
  # the mean geno is no longer 2*maf*(1-maf)/N either.

  # Aside from a few intermediary variables (below), the 
  # commented expression for sigma is the general formula for 
  # the standard deviation of the numerator of the trend 
  # statistic when HWE does not hold...
  # nhom1 <- ?
  # nhet  <- ?
  # x     <- (2*nhom1 + nhet) / (ncase + ncont)

  # sigma <- sqrt( (4*nhom1 + nhet - (ncase + ncont) * x^2) / (ncase + ncont) )

  g1      <- 4*maf^3*(1-maf)
  g2      <- 4*(1-maf)^3*maf
  g3      <- 4*maf^2*(1-maf)^2
  g       <- g1+g2+g3            # the value of g has been verified empirically

  g1      <- g1/g
  g2      <- g2/g
  g3      <- g3/g

  g.vec  <- c(g1,g2,g3)
  gammas <- sum(c(1,.5,1.5)*g.vec) - sum(c(.5,.5,1)*g.vec) * sum(c(1.5,.5,1)*g.vec)

  # A more elaborate, but equivalent formula for 'gammas'
  # gammas   <- g1+.5*g2+1.5*g3-.75*g1^2-.25*g2^2-g3^2-g1*g2-2*g1*g3-g2*g3

  niat         <- g * ncase       # the expected number of informative case-parent trios
  condcov      <- (2*niat)/(sqrt(nhp)*ncase*sigma) * gammas

  start <- trunc(nhp - sqrt(het*(1-het)*2*ncase)*3)
  end <- trunc(nhp + sqrt(het*(1-het)*2*ncase)*3)
  covzpopzfam  <- 0
  for(i in start:end)             # approx. the unconditional cov
  {
    x <- (niat/nhp) * i
    covzpopzfam  <- covzpopzfam + (2*x)/(sqrt(i)*ncase*sigma) * gammas * dbinom(i,(2*ncase),het)
  }
  return(covzpopzfam)
}

getpars <- t(matrix(scan("popfam-sim.par",what="char"),nrow=2))
alpha   <- as.numeric(getpars[1,2])
nreps   <- as.numeric(getpars[2,2])
diseq   <- as.numeric(getpars[3,2])                 # 0 <= diseq <= min(maf,daf) - daf*maf; pertains to h11 - daf*maf
daf     <- as.numeric(getpars[4,2])                 # 0 < daf < 1; The '1' allele at the disease locus is ALWAYS the 'causal allele'
maf     <- as.numeric(getpars[5,2])                 # 0 < maf < 1; If diseq > 0, then the 'risk allele' is the '1' allele
pen0    <- as.numeric(getpars[6,2])                 # penetrance for zero copies of the causal allele (i.e. the phenocopy rate)
pen1    <- as.numeric(getpars[7,2])                 # penetrance for one copy of the causal allele
pen2    <- as.numeric(getpars[8,2])                 # penetrance for two copies of the causal allele
ncase   <- as.numeric(getpars[9,2])
ncont   <- as.numeric(getpars[10,2])
save    <- as.numeric(getpars[11,2])                # binary indicator: 0 --> do NOT save ea. rep, 1 --> save ea. rep

if( diseq > min(maf,daf) - daf*maf || diseq < 0 )   # if (diseq,daf,maf) is NOT a valid triplet
{
  print(" The desired combination of LD, DAF, and MAF is not possible . ") 
}

if( diseq >= 0 && diseq <= min(maf,daf) - daf*maf ) # if (diseq,daf,maf) is valid, then continue on with the simulation...
{
  rsqd    <- diseq^2/(daf*(1-daf)*maf*(1-maf))
  dmax    <- min(daf*(1-maf), (1-daf)*maf)          # since D > 0, by the assumptions of this code 
  dprime  <- diseq/dmax   
  pen     <- c(pen0,pen1,pen2)                      # homo non-causal, het, homo causal
  gfDL    <- c((1-daf)^2,2*daf*(1-daf),daf^2)       # homo non-causal, het, homo causal: assumes HWE 
  gfSNP   <- c((1-maf)^2,2*maf*(1-maf),maf^2)       # homo non-risk, het, homo risk: assumes HWE 
  K       <- sum(pen*gfDL)                          # compute the prevalence 
  ccvals  <- NULL                                   # pvals for the case-control
  tdtvals <- NULL                                   # pvals for the tdt
  CC      <- NULL                                   # the [1 x nreps] vector of case-cont test stats
  TDT     <- NULL                                   # the [1 x nreps] vector of tdt test stats 
  WSD     <- NULL                                   # the [1 x nreps] vector of SD-based weights for Zpop from Chen & Lin 
  Fhets   <- NULL                                   # the [1 x nreps] vector of nhets/(2*ncase)
  tcrit   <- qnorm(1-alpha/2) 
  uncondcov <- uccov(maf,ncase,ncont)               # compute the uncond cov(Zpop,Zfam)
 
  # compute the conditional geno probs at the DL

  cpDLg <- (pen*gfDL)/K                             # homo non-causal, het, homo causal

  # compute the probability of allele i at the SNP given allele j at the DL

  p11 = (diseq + daf * maf)/daf
  p21 = 1 - p11
  p12 = (maf - (diseq + daf * maf))/(1 - daf)
  p22 = 1 - p12

  # compute the vector of conditional probs of genos at the SNP given the genotype at the DL; 
  # this will be used to simulate the SNP genos for controls

  ## sum to 1 check: YES

  cpsnpg22 <- c(p22^2, 2*p12*p22, p12^2)                 # cond on wildtype DL (2/2)
  cpsnpg12 <- c(p21*p22, (p11*p22 + p12*p21), p11*p12)   # cond on het DL      (1/2)
  cpsnpg11 <- c(p21^2, 2*p11*p21, p11^2)                 # cond on homo mut    (1/1)

  # compute the unconditional two-site haplotype freqs

  ## sum to 1 check: YES

  h11 <- p11 * daf
  h12 <- p12 * (1-daf)
  h21 <- p21 * daf
  h22 <- 1 - (h11 + h12 + h21)
  allhapf <- c(h11,h12,h21,h22)
 
  # compute the [10 x 55] matrix of offspring diplotype segregation probs 
  # conditional on the diplotype-based mating type. The integer codes for   
  # for the 4 possible two-site haplotypes are:
  #
  # 1 := 1-1
  # 2 := 1-2
  # 3 := 2-1
  # 4 := 2-2
  #
  # and, the key for the 10 possible integer coded diplotypes:
  #
  #  1 := 1-1/1-1 = (1*1)/(1+1)*420 = 210
  #  2 := 1-1/1-2 = (1*2)/(1+2)*420 = 280
  #  3 := 1-1/2-1 = (1*3)/(1+3)*420 = 315
  #  4 := 1-1/2-2 = (1*4)/(1+4)*420 = 336
  #  5 := 1-2/1-2 = (2*2)/(2+2)*420 = 420
  #  6 := 1-2/2-1 = (2*3)/(2+3)*420 = 504
  #  7 := 1-2/2-2 = (2*4)/(2+4)*420 = 560
  #  8 := 2-1/2-1 = (3*3)/(3+3)*420 = 630
  #  9 := 2-1/2-2 = (3*4)/(3+4)*420 = 720
  # 10 := 2-2/2-2 = (4*4)/(4+4)*420 = 840 

  icd <- c(210,280,315,336,420,504,560,630,720,840)   # two-locus diplotypes
  icdinv <- matrix(rep(0,1680),nrow=840)
  icdinv[210,] <- c(1,1)
  icdinv[280,] <- c(1,2)
  icdinv[315,] <- c(1,3)
  icdinv[336,] <- c(1,4)
  icdinv[420,] <- c(2,2)
  icdinv[504,] <- c(2,3)
  icdinv[560,] <- c(2,4)
  icdinv[630,] <- c(3,3)
  icdinv[720,] <- c(3,4)
  icdinv[840,] <- c(4,4)
 
  icdm <- matrix(rep(0,110), nrow=55) # the elements of this arrary are two-locus diplotypes
  icdm[1,]  <- c(1,1);  icdm[12,] <- c(2,3);  icdm[23,] <- c(3,6);  icdm[34,] <- c(4,10); icdm[45,] <- c(6,10) 
  icdm[2,]  <- c(1,2);  icdm[13,] <- c(2,4);  icdm[24,] <- c(3,7);  icdm[35,] <- c(5,5);  icdm[46,] <- c(7,7)
  icdm[3,]  <- c(1,3);  icdm[14,] <- c(2,5);  icdm[25,] <- c(3,8);  icdm[36,] <- c(5,6);  icdm[47,] <- c(7,8)
  icdm[4,]  <- c(1,4);  icdm[15,] <- c(2,6);  icdm[26,] <- c(3,9);  icdm[37,] <- c(5,7);  icdm[48,] <- c(7,9)
  icdm[5,]  <- c(1,5);  icdm[16,] <- c(2,7);  icdm[27,] <- c(3,10); icdm[38,] <- c(5,8);  icdm[49,] <- c(7,10)
  icdm[6,]  <- c(1,6);  icdm[17,] <- c(2,8);  icdm[28,] <- c(4,4);  icdm[39,] <- c(5,9);  icdm[50,] <- c(8,8)
  icdm[7,]  <- c(1,7);  icdm[18,] <- c(2,9);  icdm[29,] <- c(4,5);  icdm[40,] <- c(5,10); icdm[51,] <- c(8,9)
  icdm[8,]  <- c(1,8);  icdm[19,] <- c(2,10); icdm[30,] <- c(4,6);  icdm[41,] <- c(6,6);  icdm[52,] <- c(8,10)
  icdm[9,]  <- c(1,9);  icdm[20,] <- c(3,3);  icdm[31,] <- c(4,7);  icdm[42,] <- c(6,7);  icdm[53,] <- c(9,9)
  icdm[10,] <- c(1,10); icdm[21,] <- c(3,4);  icdm[32,] <- c(4,8);  icdm[43,] <- c(6,8);  icdm[54,] <- c(9,10)
  icdm[11,] <- c(2,2);  icdm[22,] <- c(3,5);  icdm[33,] <- c(4,9);  icdm[44,] <- c(6,9);  icdm[55,] <- c(10,10)

  ## sum to 1 check: YES

  Dseg <- matrix(rep(0,550), nrow=10)      # the diplotype-based segregation matrix 
  dmap <- function(a,b){ (a*b*420)/(a+b) } # maps distinct diplotypes to distinct integers

  for(i in 1:10)                           # for ea. offspring diplotype
  {
    dip <- icd[i]                          # integer coded diplotype of the ith individual
    for(j in 1:55)                         # for ea. diplotype-based mating type
    {
      prob <- 0
      par1 <- icd[icdm[j,1]]               # the diplotype of one parent 
      par2 <- icd[icdm[j,2]]               # the diplotype of the other parent
      r <- icdinv[par1,1]                  # one haplotype of one parent
      s <- icdinv[par1,2]                  # the other haplotype of that parent
      t <- icdinv[par2,1]                  # one haplotype of the other parent
      v <- icdinv[par2,2]                  # the other haplotype of this parent

      if(dip == dmap(r,t)){ prob <- prob + 0.25 }
      if(dip == dmap(r,v)){ prob <- prob + 0.25 }
      if(dip == dmap(s,t)){ prob <- prob + 0.25 }
      if(dip == dmap(s,v)){ prob <- prob + 0.25 }
      Dseg[i,j] <- prob                    # compute Pr( i | j )
    }
  }
 
  # compute the [45 x 9] matrix of two-locus genotype-based mating type (TLGmat) 
  # probabilities given the previously sampled two-locus (i.e. SNP and DL) genotype of 
  # the offspring. 
  #
  # The single locus genotype integer codes are:
  #
  # 1 := 1/1
  # 2 := 1/2
  # 3 := 2/2
  #
  
  icslg <- c(11, 12, 22) 
 
  # The key for the 9 possible integer coded two-locus genotypes:
  #
  #  1 := 1/1,1/1 := 1111
  #  2 := 1/1,1/2 := 1112
  #  3 := 1/1,2/2 := 1122
  #  4 := 1/2,1/1 := 1211
  #  5 := 1/2,1/2 := 1212
  #  6 := 1/2,2/2 := 1222
  #  7 := 2/2,1/1 := 2211
  #  8 := 2/2,1/2 := 2212
  #  9 := 2/2,2/2 := 2222

  ictlg <- c(1111, 1112, 1122, 1211, 1212, 1222, 2211, 2212, 2222)   # two-locus genotypes 
  ictlginv <- matrix(rep(0,4444),nrow=2222)
  ictlginv[1111,] <- c(1,1)
  ictlginv[1112,] <- c(1,2)
  ictlginv[1122,] <- c(2,2)
  ictlginv[1211,] <- c(1,3)
  ictlginv[1212,] <- 0                # since the dhet is compatible w/ two diff diplotypes, it's handled separately
  ictlginv[1222,] <- c(2,4)
  ictlginv[2211,] <- c(3,3)
  ictlginv[2212,] <- c(3,4)
  ictlginv[2222,] <- c(4,4)

  ictlgm <- matrix(rep(0,90),nrow=45) # the elements of this arrary are two-locus genos
  ictlgm[1,]  <- c(1,1);  ictlgm[11,] <- c(2,3);  ictlgm[21,] <- c(3,6);  ictlgm[31,] <- c(5,5);  ictlgm[41,] <- c(7,8)
  ictlgm[2,]  <- c(1,2);  ictlgm[12,] <- c(2,4);  ictlgm[22,] <- c(3,7);  ictlgm[32,] <- c(5,6);  ictlgm[42,] <- c(7,9)
  ictlgm[3,]  <- c(1,3);  ictlgm[13,] <- c(2,5);  ictlgm[23,] <- c(3,8);  ictlgm[33,] <- c(5,7);  ictlgm[43,] <- c(8,8)
  ictlgm[4,]  <- c(1,4);  ictlgm[14,] <- c(2,6);  ictlgm[24,] <- c(3,9);  ictlgm[34,] <- c(5,8);  ictlgm[44,] <- c(8,9)
  ictlgm[5,]  <- c(1,5);  ictlgm[15,] <- c(2,7);  ictlgm[25,] <- c(4,4);  ictlgm[35,] <- c(5,9);  ictlgm[45,] <- c(9,9)
  ictlgm[6,]  <- c(1,6);  ictlgm[16,] <- c(2,8);  ictlgm[26,] <- c(4,5);  ictlgm[36,] <- c(6,6)
  ictlgm[7,]  <- c(1,7);  ictlgm[17,] <- c(2,9);  ictlgm[27,] <- c(4,6);  ictlgm[37,] <- c(6,7)
  ictlgm[8,]  <- c(1,8);  ictlgm[18,] <- c(3,3);  ictlgm[28,] <- c(4,7);  ictlgm[38,] <- c(6,8)
  ictlgm[9,]  <- c(1,9);  ictlgm[19,] <- c(3,4);  ictlgm[29,] <- c(4,8);  ictlgm[39,] <- c(6,9)
  ictlgm[10,] <- c(2,2);  ictlgm[20,] <- c(3,5);  ictlgm[30,] <- c(4,9);  ictlgm[40,] <- c(7,7)

  TLGmat <- matrix(rep(0,405),nrow=45)

  dmm <- matrix(rep(0,16),nrow=4)     # there are at most 4 sets of paired diplotypes (i.e. 4 haplotypes)    
                                      # that are consistent with any two-locus genotype-based mating type

  for(j in 1:9)                       # for ea. two-locus offspring genotype: e.g. (A/A,A/B) 
  {
    ## sum to 1 check: YES

    if( ictlg[j] == 1111 ){ denom <- cpsnpg11[3] * gfDL[3] }
    if( ictlg[j] == 1112 ){ denom <- cpsnpg12[3] * gfDL[2] }
    if( ictlg[j] == 1122 ){ denom <- cpsnpg22[3] * gfDL[1] }
    if( ictlg[j] == 1211 ){ denom <- cpsnpg11[2] * gfDL[3] }
    if( ictlg[j] == 1212 ){ denom <- cpsnpg12[2] * gfDL[2] }
    if( ictlg[j] == 1222 ){ denom <- cpsnpg22[2] * gfDL[1] }
    if( ictlg[j] == 2211 ){ denom <- cpsnpg11[1] * gfDL[3] }
    if( ictlg[j] == 2212 ){ denom <- cpsnpg12[1] * gfDL[2] }
    if( ictlg[j] == 2222 ){ denom <- cpsnpg22[1] * gfDL[1] }

    for(i in 1:45)                        # for ea. two-locus genotype-based mating type: e.g. (A/A,A/B) x (A/B,B/B) 
    {
                                          # find the diplotype-based mating type(s) with non-zero prob

      par1 <- ictlg[ictlgm[i,1]]          # the 4-digit two-locus genotype of one parent
      par2 <- ictlg[ictlgm[i,2]]          # the 4-digit two-locus genotype of the other parent
      if( par1 != 1212 && par2 != 1212 )  # neither parent is a dhet
      {
        dmm[1,] <- c(ictlginv[par1,], ictlginv[par2,]) 
        ncdm <- 1 
      } 
      if( par1 != 1212 && par2 == 1212 )  # one parent is NOT a double het, but the other is
      {
        dmm[1,] <- c(ictlginv[par1,],1,4)                      
        dmm[2,] <- c(ictlginv[par1,],2,3)                      
        ncdm <- 2
      }
      if( par1 == 1212 && par2 != 1212 )  # one parent is a double het, but the other is NOT
      {
        dmm[1,] <- c(1,4,ictlginv[par2,])
        dmm[2,] <- c(2,3,ictlginv[par2,])
        ncdm <- 2
      }
      if( par1 == 1212 && par2 == 1212 )  # both parents are double hets
      {
        dmm[1,] <- c(1,4,1,4)
        dmm[2,] <- c(1,4,2,3)
        dmm[3,] <- c(2,3,2,3)
        ncdm <- 3
      }

      myprob <- 0
      for(k in 1:ncdm)                    # for ea. consistent diplotype-based mating type
      {
        r <- dmm[k,1]                     # one haplotype of one parent
        s <- dmm[k,2]                     # the other haplotype of that parent
        t <- dmm[k,3]                     # one haplotype of the other parent
        v <- dmm[k,4]                     # the other haplotype of this parent

                                          # compute the uncond prob of that particular diplotype-based mating type 
                                          # note that, when summed over all 55 possible diplotype-based mating types, 
                                          # pdmat did indeed sum to 1 (10/30/12).

        ## sum to 1 check: YES (see note above)

        dip1 <- allhapf[r] * allhapf[s]
        if( r != s ){ dip1 <- 2 * allhapf[r] * allhapf[s] }
        dip2 <- allhapf[t] * allhapf[v]
        if( t != v ){ dip2 <- 2 * allhapf[t] * allhapf[v] }
        pdmat <- 2 * dip1 * dip2                                      # both dips are different
        if( r == t && s == v ){ pdmat <- dip1 * dip2 }                # both dips are the same

                                                                      # get the inputs for Dseg[.,.], and remember that there may be two offspring 
                                                                      # diplotypes that are consistent with the obs'd two-locus offspring genotype

        dpar1 <- which(icd == dmap(r,s))                              # integer coded [1-10] diplo of par1
        dpar2 <- which(icd == dmap(t,v))                              # integer coded [1-10] diplo of par2 
        kk <- min( dpar1, dpar2) 
        dpar2 <- max( dpar1, dpar2)
        dpar1 <- kk
        flag <-0 
        kk <- 1
        while(kk <= 55 && flag == 0)                                  # find the integer coded [1-55] diplotype-based mating type for dpar1 and dpar2
        { 
          if(dpar1 == icdm[kk,1] && dpar2 == icdm[kk,2])
          { 
            dmat <- kk 
            flag <- 1
          }
          kk <- kk + 1
        }

        if( ictlg[j] != 1212 )                                        # offspring is NOT a double het
        {
          kid <- ictlg[j]                                             # the 4-digit two-locus geno of the kid
          x <- dmap(ictlginv[kid,1], ictlginv[kid,2])                 # his/her (3-digit) integer coded diplo
          dkid <- which( icd == x)                                    # his/her integer coded [1-10] diplo
          dseg <- Dseg[dkid,dmat]                                     # Pr( kid diplo | diplotype-based mating type )
        }        
        if( ictlg[j] == 1212 ){ dseg <- Dseg[4,dmat] + Dseg[6,dmat] } # offspring is a double het
        myprob <- myprob + (dseg * pdmat)
      }
      TLGmat[i,j] <- myprob / denom 
    }
  }

  # FOR loop on nreps  
  
  for( k in 1:nreps)
  {
  
    # simulate control genos (assuming HWE) at the DL and at the SNP

    contsDL <- NULL
    for(i in 1:ncont)
    {
      u <- runif(1,0,1)
      if( u < gfDL[1] ){ contsDL <- c(contsDL, 0) }
      if( u >= gfDL[1] && u < sum(gfDL[1:2]) ){ contsDL <- c(contsDL, 1/2) }
      if( u >= sum(gfDL[1:2]) ){ contsDL <- c(contsDL, 1) }
    }
    contsSNP <- NULL
    for(i in 1:ncont)
    {
      u <- runif(1,0,1)
      if( u < gfSNP[1] ){ contsSNP <- c(contsSNP, 0) }
      if( u >= gfSNP[1] && u < sum(gfSNP[1:2]) ){ contsSNP <- c(contsSNP, 1/2) }
      if( u >= sum(gfSNP[1:2]) ){ contsSNP <- c(contsSNP, 1) }
    }

    # simulate case genos at the DL given the trait model

    casesDL <- NULL
    for(i in 1:ncase)
    {
      u <- runif(1,0,1)
      if( u < cpDLg[1] ){ casesDL <- c(casesDL, 0) } 
      if( u >= cpDLg[1] && u < sum(cpDLg[1:2]) ){ casesDL <- c(casesDL, 1/2) }
      if( u >= sum(cpDLg[1:2]) ){ casesDL <- c(casesDL, 1) }
    }

    casesSNP <- NULL
    # simulate case genos at the SNP given genos at the DL 
 
    for(i in 1:ncase)
    {
      u <- runif(1,0,1)
      if( casesDL[i] == 0 )                                # if this case is 2/2 at the DL
      {
        if( u < cpsnpg22[1] ){ casesSNP <- c(casesSNP, 0) }
        if( u >= cpsnpg22[1] && u < sum(cpsnpg22[1:2]) ){ casesSNP <- c(casesSNP, 1/2) }
        if( u >= sum(cpsnpg22[1:2]) ){ casesSNP <- c(casesSNP, 1) }
      }
      if( casesDL[i] == 1/2 )                              # if this case is 1/2 at the DL
      {
        if( u < cpsnpg12[1] ){ casesSNP <- c(casesSNP, 0) }
        if( u >= cpsnpg12[1] && u < sum(cpsnpg12[1:2]) ){ casesSNP <- c(casesSNP, 1/2) }
        if( u >= sum(cpsnpg12[1:2]) ){ casesSNP <- c(casesSNP, 1) }
      }
      if( casesDL[i] == 1 )                                # if this case is 1/1 at the DL
      {
        if( u < cpsnpg11[1] ){ casesSNP <- c(casesSNP, 0) }
        if( u >= cpsnpg11[1] && u < sum(cpsnpg11[1:2]) ){ casesSNP <- c(casesSNP, 1/2) }
        if( u >= sum(cpsnpg11[1:2]) ){ casesSNP <- c(casesSNP, 1) }
      }
    }
    
    tlmatetyp <- NULL

    # simulate two-locus mating types for the parents of cases given the two-locus geno of each case 

    nhets <- 0                                                # number of het parents
    ntoa  <- 0                                                # number of transmitted '1' alleles 
    for(i in 1:ncase)                                         # for each case
    {
      g1 <- 3 - 2*casesSNP[i]
      g2 <- 3 - 2*casesDL[i]
      x <- (icslg[g1] * 100) + icslg[g2]                      # the 4-digit two-locus geno of the ith case   
      casegg <- which( ictlg == x )                           # the corresponding single digit interger code
      snpgeno <- trunc(x/100)                                 # the double digit SNP genotype of the ith case  

      u <- runif(1,0,1)
      flag <- 0 
      j <- 1
      lbracket <- 0 
      rbracket <- TLGmat[1,casegg]
      while(j <= 45 && flag == 0)                             # sample a two-locus mating type conditional on 'casegg'      
      {
        if( u >= lbracket && u < rbracket )                   # score this case-parent trio
        {
          tlmatetyp <- c(tlmatetyp,j)
          flag <- 1

          tg1   <- trunc(ictlg[ictlgm[j,1]]/100)
          tg2   <- trunc(ictlg[ictlgm[j,2]]/100)
          slmat <- tg1*100 + tg2                              # by design, tg1 is always less than or equal to tg2 

          if( slmat == 1112 )
          {
            nhets <- nhets + 1
            if( snpgeno == 11 ){ ntoa <- ntoa + 1 }
          }
          if( slmat == 1222 )
          {
            nhets <- nhets + 1
            if( snpgeno == 12 ){ ntoa <- ntoa + 1 }
          }
          if( slmat == 1212 )
          {
            nhets <- nhets + 2
            if( snpgeno == 11 ){ ntoa <- ntoa + 2 }
            if( snpgeno == 12 ){ ntoa <- ntoa + 1 }
          }
        }
        if( u > rbracket )                                   # redefine lbracket and rbracket
        {
          j <- j + 1
          lbracket <- rbracket 
          rbracket <- rbracket + TLGmat[j,casegg]
        }
      }
    }

    # output each replicate
    if(save == 1)
    {
      file <- paste("POP/cont-",k,".dat",sep="")
      write("CONTROL-ID GENOTYPE",file)
      for(j in 1:ncont)
      {
        if(contsSNP[j]==0){geno <- c(2,2)}
        if(contsSNP[j]==0.5){geno <- c(1,2)}
        if(contsSNP[j]==1){geno <- c(1,1)}
        line <- paste("",j,geno[1],geno[2])
        write(line,file,append=T)
      }
      file <- paste("FAM/fam-",k,".dat",sep="")
      write("FAM-ID SUB-ID FATH-ID MOTH-ID SEX AFFECTION GENOTYPE",file)
      for(j in 1:ncase)
      {
        jj   <- tlmatetyp[j]                   # this is the two-locus mating type for the jth case
        tg1  <- trunc(ictlg[ictlgm[jj,1]]/100) # dad's double digit genotype
        tg2  <- trunc(ictlg[ictlgm[jj,2]]/100) # mom's double digit genotype

        if(tg1==11){dgeno <- c(1,1)}
        if(tg1==12){dgeno <- c(1,2)}
        if(tg1==22){dgeno <- c(2,2)}

        if(tg2==11){mgeno <- c(1,1)}
        if(tg2==12){mgeno <- c(1,2)}
        if(tg2==22){mgeno <- c(2,2)}

        if(casesSNP[j]==0){kgeno <- c(2,2)}
        if(casesSNP[j]==0.5){kgeno <- c(1,2)}
        if(casesSNP[j]==1){kgeno <- c(1,1)}
        lined <- paste("",j,"1 0 0 1 0",dgeno[1],dgeno[2])
        linem <- paste("",j,"2 0 0 2 0",mgeno[1],mgeno[2])
        linek <- paste("",j,"3 1 2 1 2",kgeno[1],kgeno[2])
        write(lined,file,append=T)
        write(linem,file,append=T)
        write(linek,file,append=T)
      }
    }

    # compute Zpop (i.e. trend)  and Zfam (i.e. tdt) for this replicate

    gamma <- sqrt( var(casesSNP)/ncase + var(contsSNP)/ncont)
    cc  <- (mean(casesSNP) - mean(contsSNP)) / gamma
    CC  <- c(CC,cc)
    ccvals <- c(ccvals, 2*(1-pnorm(abs(cc))))

    lambda <- 1 / (2 * sqrt(nhets)) 
    tdt <- (ntoa/nhets - 1/2) / lambda 
    TDT <- c(TDT,tdt) 
    tdtvals <- c(tdtvals, 2*(1-pnorm(abs(tdt))))

    # here are the Chen & Lin weights

    wsd <- (lambda * (1-gamma*uncondcov)) / (lambda * (1-gamma*uncondcov) + gamma * (1-lambda*uncondcov) )
    WSD <- c(WSD,wsd)

    Fhets <- c(Fhets, nhets/(2*ncase)) 
  }

  muzpop <- mean(CC)               # this is used when finding the optimal weigth
  muzfam <- mean(TDT)              # this is used when finding the optimal weight
  wseq  <- seq(0.01,.99,.01)       # set up a grid search to find the optimal weight 
  theta <- ( wseq * muzpop + (1-wseq) * muzfam ) / sqrt( wseq^2 + (1-wseq)^2 + 2*wseq*(1-wseq)*uncondcov )
  wopt  <- ( which.max(1 - pnorm(tcrit - theta) + pnorm(-tcrit - theta)) / (length(wseq) + 1) )
  zmixopt  <- (wopt*CC + (1-wopt)*TDT) / sqrt( wopt^2 + (1-wopt)^2 + 2*wopt*(1-wopt)*uncondcov )
  zmixoptvals <- 2*(1-pnorm(abs(zmixopt)))

  zmixsd  <- ( wsd*CC + (1-wsd)*TDT ) / sqrt( WSD^2 + (1-WSD)^2 + 2*WSD*(1-WSD)*uncondcov )
  zmixsdvals <- 2*(1-pnorm(abs(zmixsd)))

  ccpow <- round(sum(ccvals <= alpha)/nreps, 3)
  tdtpow  <- round(sum(tdtvals <= alpha)/nreps, 3)
  zmixoptpow <- round(sum(zmixoptvals <= alpha)/nreps, 3)
  zmixsdpow <- round(sum(zmixsdvals <= alpha)/nreps, 3)

  write(" ","popfam-sim.out")
  write("========================================", "popfam-sim.out", append=T)
  write( paste(" Ncases, Nconts, Nreps: (", ncase, ncont, nreps,")"), "popfam-sim.out", append=T )
  write( paste(" Prevalence in the population:", round(K,3)), "popfam-sim.out", append=T )
  write( paste(" DL-SNP correlation (i.e. rsqd):", round(rsqd,3)), "popfam-sim.out", append=T ) 
  write( paste(" D-prime btwn DL and SNP:", round(dprime,3)), "popfam-sim.out", append=T )
  write( paste(" Cov(CC,TDT):", round(uncondcov,3)), "popfam-sim.out", append=T )
  write("========================================", "popfam-sim.out", append=T)
  write(" ", "popfam-sim.out", append=T)
  write( paste("         ZMIX power:", zmixsdpow), "popfam-sim.out", append=T )
  write(" ", "popfam-sim.out", append=T)
  write("      --------------------", "popfam-sim.out", append=T)
  write( paste("          TDT power:", tdtpow), "popfam-sim.out", append=T ) 
  write( paste("           CC power:", ccpow), "popfam-sim.out", append=T )
  write( paste("      Maximum power:", zmixoptpow), "popfam-sim.out", append=T )
  write("      ====================", "popfam-sim.out", append=T)
  write(" ", "popfam-sim.out", append=T)
  write("...THANK YOU for using POPFAM-v1.0", "popfam-sim.out", append=T)
  write(" ", "popfam-sim.out", append=T)
}
 
