# The purpose of 'popfam-test.r' is to compute Zmix from a 
# case-control/case-parent data set, wherein some (possibly 
# all) of the cases are shared between the population-based 
# and family-based designs. In addition, 'popfam-test.r' also 
# computes the p-value associated with Zmix, as well as the 
# corresponding test statistics and p-values for Zpop (e.g. 
# trend) and Zfam (e.g. TDT).

# This function returns the unconditional covariance btwn 
# Zpop and Zfam.
uccov <- function(maf, ncase, ncont)
{
  het   <- 2*maf*(1-maf)
  nhp   <- het * 2 * ncase

  # Note: the Trend test compares the difference in expected dose. So,
  # genos are coded as 0, 1, and 2, and when there is random mating,
  # each geno ~ Bin(2,maf) w/ mean 2*maf, and variance 2*maf*(1-maf).
  # Hence, the variance of the mean geno (which estimates the
  # expected dose) is 2*maf*(1-maf)/N, where N is ncase for cases,
  # and ncont for controls. Under HWE, the Trend test is the same
  # as the test that compares allele freqs btwn cases and controls.

  sigma <- sqrt(2 * maf*(1-maf)*( 1/ncase + 1/ncont ) )

  # When HWE does not hold, the distribution of each geno is no
  # longer Bin(2,maf), because the component alleles are no longer
  # independent Ber(maf) rv's. Needless to say, the variance of
  # the mean geno is no longer 2*maf*(1-maf)/N either.

  # Aside from a few intermediary variables (below), the 
  # commented expression for sigma is the general formula for 
  # the standard deviation of the numerator of the trend 
  # statistic when HWE does not hold...
  # nhom1 <- ?
  # nhet  <- ?
  # x     <- (2*nhom1 + nhet) / (ncase + ncont)

  # sigma <- sqrt( (4*nhom1 + nhet - (ncase + ncont) * x^2) / (ncase + ncont) )

  g1      <- 4*maf^3*(1-maf)
  g2      <- 4*(1-maf)^3*maf
  g3      <- 4*maf^2*(1-maf)^2
  g       <- g1+g2+g3            # the value of g has been verified empirically

  g1      <- g1/g
  g2      <- g2/g
  g3      <- g3/g

  g.vec  <- c(g1,g2,g3)
  gammas <- sum(c(1,.5,1.5)*g.vec) - sum(c(.5,.5,1)*g.vec) * sum(c(1.5,.5,1)*g.vec)

  # A more elaborate, but equivalent formula for 'gammas'
  # gammas   <- g1+.5*g2+1.5*g3-.75*g1^2-.25*g2^2-g3^2-g1*g2-2*g1*g3-g2*g3

  niat         <- g * ncase       # the expected number of informative case-parent trios
  condcov      <- (2*niat)/(sqrt(nhp)*ncase*sigma) * gammas

  start <- trunc(nhp - sqrt(het*(1-het)*2*ncase)*3)
  end <- trunc(nhp + sqrt(het*(1-het)*2*ncase)*3)
  covzpopzfam  <- 0
  for(i in start:end)             # approx. the unconditional cov
  {
    x <- (niat/nhp) * i
    covzpopzfam  <- covzpopzfam + (2*x)/(sqrt(i)*ncase*sigma) * gammas * dbinom(i,(2*ncase),het)
  }
  return(covzpopzfam)
}

# read in the arguments for popfam-test.r

getpars  <- t(matrix(scan("popfam-test.par",what="char"),nrow=2))
alpha    <- as.numeric(getpars[1,2])                 # the (apriori) fixed type I error 
contfile <- getpars[2,2]                             # name of the file that contains control data
famfile  <- getpars[3,2]                             # name of the file that contains case-parent data
maf      <- as.numeric(getpars[4,2])                 # minor allele freq (maf) of the SNP

# read in the relevant data

contfile <- paste("POP/",contfile,sep="")
famfile  <- paste("FAM/",famfile,sep="")
cont.mat <- t(matrix(scan(contfile,skip=1),nrow=3)) 
fam.mat  <- t(matrix(scan(famfile,skip=1),nrow=8))
ncase    <- nrow(fam.mat)/3
ncont    <- nrow(cont.mat) 

# compute Zpop (i.e. trend)  and Zfam (i.e. tdt) for this replicate

contsSNP <- 2 - 0.5 * apply(cont.mat[,2:3],1,sum)   # consistenty check: YES 11/14/2012
x <- NULL
for(i in 1:ncase){ x <- rbind(x, fam.mat[(3*i),7:8] ) }
casesSNP <- 2 - 0.5 * apply(x,1,sum)
gamma <- sqrt( var(casesSNP)/ncase + var(contsSNP)/ncont)
cc  <- (mean(casesSNP) - mean(contsSNP)) / gamma    # agrees with 'simcctrio.r' on my office Mac
ccpval <- 2*(1-pnorm(abs(cc)))

nhets <- 0 
ntoa  <- 0
for(i in 1:ncase)
{
  f <- 3*i - 2
  m <- 3*i - 1 
  k <- 3*i
  slmat <- 1000*fam.mat[f,7] + 100*fam.mat[f,8] + 10*fam.mat[m,7] + fam.mat[m,8]
  snpgeno <- 10*fam.mat[k,7] + fam.mat[k,8]
  if( slmat == 1112 )
  {
    nhets <- nhets + 1
    if( snpgeno == 11 ){ ntoa <- ntoa + 1 }
  }
  if( slmat == 1222 )
  {
    nhets <- nhets + 1
    if( snpgeno == 12 ){ ntoa <- ntoa + 1 }
  }
  if( slmat == 1212 )
  {
    nhets <- nhets + 2
    if( snpgeno == 11 ){ ntoa <- ntoa + 2 }
    if( snpgeno == 12 ){ ntoa <- ntoa + 1 }
  } 
}

lambda <- 1 / (2 * sqrt(nhets))
tdt <- (ntoa/nhets - 1/2) / lambda                # agrees with 'simcctrio.r' on my office Mac
tdtpval <- 2*(1-pnorm(abs(tdt)))

# compute Zmix using the Chen & Lin weights

uncondcov <- uccov(maf,ncase,ncont)               # compute the uncond cov(Zpop,Zfam)
wsd <- (lambda * (1-gamma*uncondcov)) / (lambda * (1-gamma*uncondcov) + gamma * (1-lambda*uncondcov) )
zmixsd  <- ( wsd*cc + (1-wsd)*tdt ) / sqrt( wsd^2 + (1-wsd)^2 + 2*wsd*(1-wsd)*uncondcov )
zmixsdpval <- 2*(1-pnorm(abs(zmixsd)))

sig <- 4
write(" ","popfam-test.out")
write("========================================", "popfam-test.out", append=T)
write( paste(" Ncases, Nconts, MAF: (", ncase, ncont, maf,")"), "popfam-test.out", append=T )
write( paste(" Cov(CC,TDT):", round(uncondcov,sig)), "popfam-test.out", append=T )
write("========================================", "popfam-test.out", append=T)
write(" ", "popfam-test.out", append=T)
write( paste("         ZMIX :", round(zmixsd,sig)), "popfam-test.out", append=T )
write( paste("      p-value :", round(zmixsdpval,sig)), "popfam-test.out", append=T )
write(" ", "popfam-test.out", append=T)
write("      ---------------------------", "popfam-test.out", append=T)
write( paste("          TDT :", round(tdt,sig), "(", round(tdtpval,sig), ")"), "popfam-test.out", append=T )
write( paste("           CC :", round(cc,sig), "(", round(ccpval,sig), ")"), "popfam-test.out", append=T )
write("      ===========================", "popfam-test.out", append=T)
write(" ", "popfam-test.out", append=T)
write("...THANK YOU for using POPFAM-v1.0", "popfam-test.out", append=T)
write(" ", "popfam-test.out", append=T)

