/* This collection of sources checks command-line    */
/* for existence and accuracy, prints summary        */
/* statistics, frees memory, and reads in the        */
/* original, dense SNP linkage data.                 */
/*                                                   */
/*           The functions in this file are:         */
/*                1. pr_opts()                       */
/*                2. errchk(a,b,)                    */
/*                3. pr_sumstats(a,b,c)              */
/*                4. freemem(a)                      */
/*                5. readped(a,b)                    */
/*                6. readdat(a,b)                    */
/*                7. writedat(a,b,c,d)               */


#include <stdlib.h>
#include <stdio.h>
#include "../Headers/globs.h"
#include "../Headers/ldsubsample.h"


/*************************/
/* Print program options */
/*************************/

void pr_opts ()
{
  system("clear");
  printf("\n  LDsubsample_v1.0 (2009) : William C. L. Stewart, NCH/OSU \n");
  printf(  "            Reference : Stewart et al (2009) EJHG 19:1090--94 \n");
  printf("\n      Program options : [pedfile] [datfile] [mapfile] [LD-measure] [LD-limit] [Het-limit] [Nsubs] [seed] [Nskip]\n");
  printf(  "       Example syntax : gold.ped   gold.dat  gold.map     rsq        .05         .3         10    062070   100\n\n");
}


/***************************************************/
/* Check for command-line argument errors. If an   */
/* error exists, then exit prematurely with the    */
/* corresponding error message (written to stderr) */
/***************************************************/

void errchk( char** args )
{
  if( args[1] == NULL )
  {
    fprintf(stderr,"\nError : [pedfile] not specified. \n\n");
    pr_opts();
    exit(8);
  }
  if( args[2] == NULL )
  {
    fprintf(stderr,"\nError : [datfile] not specified. \n\n");
    pr_opts();
    exit(8);
  }
  if( args[3] == NULL )
  {
    fprintf(stderr,"\nError : [mapfile] not specified. \n\n");
    pr_opts();
    exit(8);
  }
  if( strcmp(args[4],"rsq")!=0 && strcmp(args[4],"dprime")!=0 )
  {
    fprintf(stderr,"\nError : %s is not allowed; choose either rsq or dprime.\n", args[4]);
    pr_opts();
    exit(8);
  }
  if( atof(args[5]) <= 0 || atof(args[5]) > 1)
  {
    fprintf(stderr,"\nError : Threshold for LD measure must be in (0,1]. \n");
    pr_opts();
    exit(8);
  }
  //if( atof(args[6]) < 0 || atof(args[6]) > 0.5)
  //{ 
  //  fprintf(stderr,"\nError : Threshold for heterozygosity must be in [0,0.5] \n");
  //  pr_opts();
  //  exit(8);
  //}
  if( atoi(args[7]) < 1 )
  {
    fprintf(stderr,"\nError : Number of RBSNP replicates must be greater than or equal to 1 \n");
    pr_opts();
    exit(8);
  }
}


/****************************************************/
/* Print subsample summary statistics out to a file */
/****************************************************/

void pr_sumstats( int* dim, char** args, float asso, int tot, int nsubs)
{
  char file[100];
  float avg;
  FILE* fptr;
 
  sprintf(file, "%s", "Results/Subsample/sumstats.out-");
  strcat(file,  args[10]);
  fptr = fopen(file, "w");

  fprintf( fptr, "\n SUMMARY STATISTICS\n");
  fprintf( fptr, " ==================\n\n");
  avg = (double)tot / (double)nsubs; 
  fprintf( fptr, "Average number of SNPs per LE subsample:  %6.0f\n", avg);

#if 0
  if( !strcmp(args[4], "rsq") )
    fprintf( fptr, "Average R-squared is : %.4f \n", asso);
  else fprintf(fptr, "Average D-prime is : %.4f \n", asso);
  fprintf( fptr, "Average heterozygosity is : Blah \n");
#endif
}


/**************************************************/
/* Free the memory allocated for global variables */
/**************************************************/

void freemem( int* dim )
{
  int i, j, nrows, ncols;
  
  nrows = dim[0]; 
  ncols = dim[1];

  
  for( i=0; i<=nrows; i++)
    free( PED[i] );
  free( PED );

  nrows = (ncols - 6)/2;
  for( i=0; i<(nrows+2); i++)
  {
    for(j=0; j<2; j++)
      free( DAT[i][j] );
    free( DAT[i] );
  }
  free( DAT );
}


/***********************************************************/
/* Read in a retangular matrix of integers, without        */
/* specifying the dimensions.  Modify by reference the     */
/* values of nrow and ncol.  Return the matrix in a        */
/* (one-indexed) arrary.                                   */
/***********************************************************/

void readped( char* file, int* dim )
{
  int nentries, c, i, j, r, x;
  FILE* fptr;
  
  r = c = nentries = 0;
  if( (fptr = fopen(file,"r")) == NULL )
  {
    fprintf(stderr,"The file : %s was not found! \n", file);
    pr_opts();
    exit(8);
  }
  else
  {
    while( !feof(fptr) )  /* count rows first */
    {
      if( (int)fgetc(fptr) == 10 ) r++;
    }
    rewind(fptr);
    while( !feof(fptr) ) /* count cols second */
    {
      fscanf(fptr,"%d",&x); 
      nentries++;
    }
    c = nentries / r;
    
    rewind(fptr);
    PED = malloc( (r+1) * sizeof( int* ) );
    for( i=0; i<=r; i++) 
      PED[i] = malloc( (c+1) * sizeof( int ));

    for( i=0; i<=r; i++)     /* read the pedfile,    */
      for( j=0; j<=c; j++)   /* which is one-indexed */
      {
        if( i==0 || j==0 ) PED[i][j] = 0;
        else fscanf(fptr, "%d", PED[i] + j);
      } 
  }
  
  dim[0] = r;
  dim[1] = nentries / r;

  fclose( fptr );
}


/******************************/
/* Read in the merlin datfile */
/******************************/

void readdat( char* file, int cols )
{
  int i, j, rowchk = 0;
  FILE* fptr;

  if( (fptr = fopen(file,"r")) == NULL )
  {
    fprintf(stderr,"The file : %s was not found! \n", file);
    pr_opts();
    exit(8);
  }
  else
  {
    while( !feof(fptr) )  /* count rows first */
    {
      if( (int)fgetc(fptr) == 10 ) rowchk++;
    }
    if( (rowchk-2) != (cols - 6)/2 )
    {
      fprintf(stderr,"Error : pedfile has %d SNPs while ",(cols-6)/2);
      fprintf(stderr,"datfile has %d SNPs.\n\n", (rowchk-2));
      exit(8);
    }

    rewind(fptr);
    DAT = malloc( rowchk * sizeof( char** ) );
    for( i=0; i<rowchk; i++)
    {
      DAT[i] = malloc( 2 * sizeof( char* ));
      for(j=0; j<2; j++)
        DAT[i][j] = malloc( 20 * sizeof( char ) );
    }

    for( i=0; i<rowchk; i++)                          /* read the datfile,      */
      fscanf(fptr, "%s%s", DAT[i][0], DAT[i][1]);     /* which is zero-indexed  */
  }
  
  fclose(fptr);
}


/****************************************************/
/* Write the new (sparse) merlin datfile to a file  */
/* This code assumes that the old (dense) merlin    */
/* datfile contains only M's                        */
/****************************************************/

void writedat( char* file, int cols, Nodeptr currptr, int useLD )
{
  int i, flag, rows, u;
  FILE* fptr;

  fptr = fopen(file, "w");
  rows = (cols - 6)/2 + 2;      /* cardinal number of rows in datfile     */

                                /* write the ldlink datfile               */
  while( currptr->nxtleftptr != NULL) currptr = currptr->nxtleftptr;

  if( !useLD ) u = (rand() % (rows-5)) + 2; /* get a random, interior SNP */
  for( i=0; i<rows; i++)
  {
    flag = 0;
    if( useLD )             /* useLD = 1 --> use pairwise estimates of LD */
    {
      if( currptr )
        if( (currptr->snpindx) == i )    /* this snp is in the subsample  */
        {
          fprintf(fptr, "%-5s %-10s \n", DAT[i][0], DAT[i][1]);
          currptr = currptr->nxtrightptr;
          flag = 1;
        }
    }
               /* can create distance based subsamples if desired         */
    else       /* if we're not using LD, graph a random interior SNP      */
    {
      if( i == u )
      {
        fprintf(fptr, "%-5s %-10s \n", DAT[i][0], DAT[i][1]);
        flag = 1;
      }
    }

    if( !flag && i > 1 && i < (rows-2))    /* snp is not in the subsample */
      fprintf(fptr, "%-5s %-10s \n", "S2 ", DAT[i][1]);

                   /* disease status and eof info are always carried over */
    if( i == 0 || i == (rows-1) )
      fprintf(fptr, "%-5s %-10s \n", DAT[i][0], DAT[i][1]);

                  /* if either the first or the last snp was not selected */ 
                  /* on the basis of LE, then select it now.              */
    if( !flag && i == 1)
      fprintf(fptr, "%-5s %-10s \n", DAT[i][0], DAT[i][1]);
    if( !flag && i == (rows-2))
      fprintf(fptr, "%-5s %-10s \n", DAT[i][0], DAT[i][1]);
  }
  fclose(fptr);
}

