#!/usr/bin/perl

## For the advanced user: see 'tutorial.pdf' for a completely worked example of EAGLET 
## in action, with detailed descriptions of input and output files.

## eaglet-v1.3
## April 5, 2011

## EAGLET : (Efficient Analysis of Genetic Linkage: Estimation and Testing) achieves  
##          improved inference by analyzing all available dense SNP genotype data, 
##          even when many of the SNPs are in high linkage disequilibrium (LD) with 
##          one another. 
##   
##          EAGLET has 6 main analysis options (see README_userdoc for details):
##
##          1. --test      : EAGLET will use multiple subsamples of the original dense 
##                           SNP data to compute a powerful test statistic for linkage.  
##          2. --estimate  : EAGLET will average estimates of trait location over 
##                           subsamples, and will compute 95% confidence intervals. 
##          3. --subsample : This allows users to subsample their dense SNP data, so that 
##                           each subsample can be analyzed with a custom linkage program.
##          4. --custom    : EAGLET will average the lod score curves generated by the 
##                           subsamples obtained from (3)
##          5. --meta      : This option combines independent LOD score curves, even if 
##                           each curve is based on different markers).
##          6. --ldinfo    : EAGLET computes LD summary statistics for dense SNP data. LD 
##                           can be measured in either rsqd or D'.
##          7. --parallel  : This option leverages multiple cores on the machine and runs is 
##			     parallel
##          
## The Results/ directory contains six subdirectories: Test, Estimate, Subsample, Custom 
## Meta, and LDinfo.  EAGLET output for any option can be found in its corresponding 
## subdirectory. 

# use warnings;
# use strict; 

## ===================================================================
## SETUP  include headers
##        declare global vars
##        check command-line args
##        if successful, print general options and analysis parameters
## ===================================================================

require 'Utils/eaglet.ph';                          ## All Perl subroutines used by EAGLET

use Getopt::Long;

$pedfile = $datfile = $mapfile = $uniqid = '';       ## REQUIRED (for test, estimate, subsample)
   $tola = $modfile = '';                           ## OPTIONAL (both neeeded for parametric analysis)
   $rldt = $dldt = '';                              ## OPTIONAL     (Pick one)
   $test = $estimate = $subsample = '';             ## OPTIONAL     (Any subset)
   $parallel = ''; 				    ## OPTIONAL     (Any subset)
   $meta = $ldinfo = '';                            ## OPTIONAL     (Any subset)
 $custom = '';                                      ## CANNOT BE USED WITH META
 $tasksize = "tiny";

$version  = "EAGLET-v2.0"; 
$wtola    = 3;                                      ## DEFAULT : Use HLODs for parametric analysis
$het      = 0.01;                                   ## DEFAULT : LB for 1st snp heterozygosity 
$nboots   = 100;                                    ## DEFAULT : number of bootstrap samples
$nskip    = 0;                                      ## DEFAULT : number of adj snps to skip 
$nsubs    = 50;                                     ## DEFAULT : number of subsamples
$posld    = 0;                                      ## DEFAULT : ignore coupling 
$yn       = "No";                                   ## DEFAULT : yes/no variable
$seed     = int(rand(9000000)) + 1000001;           ## draw a random seed between for C-functions 
$bpedfile = "boot.ped";

##          test : output the ALOD, the MAE, and the aligned constituent lods.
##        custom : computes the ALOD, AMAX, and MCE-k from custom lod score curves.
##      estimate : output the MCE-1, MCE-k, and their CIs
##     subsample : output the merlin formatted files for N random subsamples 
##        ldinfo : output the summary stats for LD info based on dense SNP data 
##          meta : output the summed LOD (based on ind. data sets) on a common grid 

$args = GetOptions ("p=s"         => \$pedfile,   ## string
                    "d=s"         => \$datfile,   ## string
		    "m=s"         => \$mapfile,   ## string
                    "model=s"     => \$modfile,   ## string
                    "para=s"      => \$tola,      ## string 
                    "ldmeasure=s" => \$ldmeasure, ## string
                    "tasksize=s"  => \$tasksize,  ## string
		    "uniqid=s"    => \$uniqid,    ## string
		    "seed=i"      => \$seed,      ## integer
                    "nsubs=i"     => \$nsubs,     ## integer
                    "nboots=i"    => \$nboots,    ## integer
                    "nskip=i"     => \$nskip,     ## integer
                    "posld"       => \$posld,     ## boolean
                    "subsample"   => \$subsample, ## boolean
                    "test"        => \$test,      ## boolean
		    "parallel"    => \$parallel,  ## boolean
                    "estimate"    => \$estimate,  ## boolean
                    "ldinfo"      => \$ldinfo,    ## boolean
                    "meta"        => \$meta,      ## boolean
                    "custom"      => \$custom,    ## boolean
                    "dldt=f"      => \$dldt,      ## decimal
                    "rldt=f"      => \$rldt);     ## decimal

$first = "";
if( $uniqid eq "" )
{ 
	$uniqid = $pedfile; 
        $first = "Yes";
}
$uniq = substr($uniqid,0,-3);
if( $tola eq "lod" || $tola eq 'mmls' ){ $wtola = 1; }
if( $posld ){ $yn = "Yes"; }
system("clear");
chkpars();
propts();    
prpars();

## ==================
## BEGIN main program
## ==================

# system("rm -f Results/Subsample/*.out*");             ## Clean up files in Subsample directory
# system("rm -f Results/Subsample/*/*");                ## and in its subdirectories
# system("rm -f Results/Test/Una*/lelod-*");            ## Clean up the files in the Test
# @a = ('alod', 'cgrid', 'clods', 'mae');               ## directory and its subdirectories
# foreach $i ( @a ){ system("rm -f Results/Test/$i"); } 
# system("rm -f Results/Estimate/sub-*");               ## Clean up files in Estimate directory
# system("rm -f Results/Estimate/*/*");                 ## and in its subdirectories

if( $subsample )  ## IF SUBSAMPLES ARE TO BE USED FOR A CUSTOM ANALYSIS
{
  `Utils/xldsubsample $pedfile $datfile $mapfile $measure $thresh $het $nsubs $seed $nskip`;
  for($i=1; $i<=$nsubs; $i++)
  {
    @subargs = ($pedfile, $mapfile, $i);
    extract( @subargs );
  }
}

if ( $test or $estimate )  ## IF TESTING or ESTIMATING w/ MERLIN
{
  print "\n";
  if( $test ) ## IF TESTING w/ MERLIN  
  {
    if( $parallel )
	{
	
	system(" cd Bashreduce; ./brscript.sh -p $pedfile -d $datfile -m $mapfile -r $rldt -n $nsubs -k $nskip -a $tola -o $modfile -s $seed -t $tasksize; cd ../");

	}
    else
	{
	if( $first eq "" ){
    	system("mkdir -p Results/Subsample/Subs-$uniq"); 
    	$ndfile = "Results/Subsample/Subs-$uniq/sub-$uniqid";
    	if( !$subsample )                                        ## If we haven't subsampled yet...
    	{ `Utils/xldsubsample $pedfile $datfile $mapfile $measure $thresh $het $nsubs $seed $nskip $ndfile`; }
    	## system("cp Results/Subsample/Subs/sub-* Results/Test/"); ## Copy subsamples        

    	print " Computing the ALOD curve based on $nsubs subsamples...\n";
    	@alodargs = ();
    	print "\n";
    	for( $i=1; $i<=$nsubs; $i++)
    	{ 
      	print " Processing subsample $i \r";        ## carriage return w/o line feed 
      	$|++;                                       ## flush the print buffer
      	if( !$tola )
      	{ 
		@lelod = `Utils/xmerlin -p Peds/$pedfile -d Results/Subsample/Subs-$uniq/sub-$uniqid -m Maps/$mapfile --npl --information`; 
        	@LELOD = getnplod( @lelod );
      	}
      	if( $tola )
      	{ 
        	@lelod = `Utils/xmerlin -p Peds/$pedfile -d Results/Subsample/Subs-$uniq/sub-$uniqid -m Maps/$mapfile --model Mods/$modfile --info`; 
        	if( $tola eq 'lod' ){ @LELOD = getplod( @lelod ); }
        	if( $tola eq 'mmls'){ @LELOD = getmmls( @lelod ); }
      	}
      	system("mkdir -p Results/Test/Unaligned-Lods-$uniq");
      	open(OUT, ">Results/Test/Unaligned-Lods-$uniq/lelod-$uniqid");     
      	print OUT @LELOD;
      	close(OUT);
      	push( @alodargs, "Results/Test/Unaligned-Lods-$uniq/lelod-$uniqid");
    	}
    	## push( @alodargs, 0 );                         ## -1 -> ESTIMATE; 0 -> ALOD; 1 -> META-LOD; 2 -> CUSTOM
    	## push( @alodargs, $uniqid ); 
    	## $MAE = alod( @alodargs );
    	## print "\n\n";
    	## print " ========   ====\n";
    	## print " Position   ALOD\n";
    	## print " ========   ====\n\n";
    	## system("paste -d ' ' Results/Test/cgrid-$uniqid Results/Test/alod-$uniqid");
    	## print "\n";
    	## system("cat Results/Test/mae-$uniqid");
    	## print "\n\n";

    	## system("rm -f Results/Test/sub-*");    
    	}
	else
	{
	
	system("mkdir -p Results/Subsample/Subs-$uniq");
        $ndfile = "Results/Subsample/Subs-$uniq/sub-$uniqid";
        if( !$subsample )                                        ## If we haven't subsampled yet...
        { `Utils/xldsubsample $pedfile $datfile $mapfile $measure $thresh $het $nsubs $seed $nskip $ndfile`; }
        	#system("cp Results/Subsample/Subs/sub-* Results/Test/"); ## Copy subsamples

        print " Computing the ALOD curve based on $nsubs subsamples...\n";
        @alodargs = ();
        print "\n";
        for( $i=1; $i<=$nsubs; $i++)
        {
        print " Processing subsample $i \r";        ## carriage return w/o line feed
        $|++;                                       ## flush the print buffer
        if( !$tola )
        {
                @lelod = `Utils/xmerlin -p Peds/$pedfile -d Results/Subsample/Subs-$uniq/sub-$uniqid -m Maps/$mapfile --npl --information`;
                @LELOD = getnplod( @lelod );
        }
        if( $tola )
        {
                @lelod = `Utils/xmerlin -p Peds/$pedfile -d Results/Subsample/Subs-$uniq/sub-$uniqid -m Maps/$mapfile --model Mods/$modfile --info`;
                if( $tola eq 'lod' ){ @LELOD = getplod( @lelod ); }
                if( $tola eq 'mmls'){ @LELOD = getmmls( @lelod ); }
        }
        system("mkdir -p Results/Test/Unaligned-Lods-$uniq");
        open(OUT, ">Results/Test/Unaligned-Lods-$uniq/lelod-$uniqid");
        print OUT @LELOD;
        close(OUT);
        push( @alodargs, "Results/Test/Unaligned-Lods-$uniq/lelod-$uniqid");
        }
        push( @alodargs, 0 );                         ## -1 -> ESTIMATE; 0 -> ALOD; 1 -> META-LOD; 2 -> CUSTOM
        push( @alodargs, $uniqid );
        $MAE = alod( @alodargs );
        print "\n\n";
        print " ========   ====\n";
        print " Position   ALOD\n";
        print " ========   ====\n\n";
        system("paste -d ' ' Results/Test/cgrid-$uniqid Results/Test/alod-$uniqid");
        print "\n";
        system("cat Results/Test/mae-$uniqid");
        print "\n\n";

	system("rm -f Results/Test/sub-*");
	}
    	}
  }

  if ( $estimate ) ## If ESTIMATING TRAIT LOCATION & CONSTRUCTING CIs 
  {
    @ped = import( "Peds/$pedfile" );  ##  SETUP vectors famsz and famst for the Bootstrap
    $currfam = "-999";
    $cfams   = 0;
    @famsz   = ();  # famsz is one-indexed
    @famst   = ();  # famst is one-indexed (the starting index of each family)
    $pedlen  = scalar(@ped);

    for($i=0; $i<$pedlen; $i++)
    {
      @words = split( " ", $ped[$i] );
      if( $words[0] ne $currfam )
      {
        $cfams++;
        $famst[$cfams] = $i;       # stores the index the first memeber of each family 
        $currfam = $words[0];
      } 
      $famsz[$cfams]++;
    }
    $famsz[0] = $cfams;
    $famst[0] = $cfams;

    ## FOR EACH BOOTSTRAP RESAMPLE AND FOR THE ORIGINAL SAMPLE, COMPUTE MCE-1, MCE-k, 
    ## and MAE (along with CIs).  THE ORIGINAL SAMPLE IS THE 0th BOOTSTRAP RESAMPLE 
    
    @MAE = @MCE1abs = @MCEk = ();       ## MCE1abs := mce-1 estimates across multiple bs resamples
    for( $i=0; $i<=($nboots); $i++ )    
    {
      $seed = srand($seed);             ## use seed to initiate a perl sequence of random #'s
      @bs = (1 .. $cfams);              ## create bootstrap resample
      if( $i >  0){ @bs  = sample_wr($cfams); }
      @bped = ();
      $uniqid = 1;                      ## use @bs to get the fams, and make their ids unique
      for($k=1; $k<=$cfams; $k++)       ## for each bs family
      { 
        $bfam   = $bs[($k-1)];          ## this is the original index of the ($k-1)th bs family in @ped
        $bfamsz = $famsz[$bfam];        ## this is the size of that family
        $bfamst = $famst[$bfam];        ## this is the index of that family in @ped
    
        for($t=$bfamst; $t<($bfamst + $bfamsz); $t++)
        {
          @rec =  split( " ", $ped[$t]);
          $rec[0] = $uniqid;
          $record = join(" ", @rec) . "\n";
          push( @bped, $record);
        }
        $uniqid++;
      }
      open( OUT, ">Peds/$bpedfile");			
      print OUT "@bped";
      close( OUT );
  
      if( $i == 0 )
      {
                                                             ## mask the fact that the 0-th 
                                                             ## iteration is the real data 
        print " Processing bootstrap resample 1 \r";         ## carriage return w/o line feed
        $|++;                                                ## flush the print buffer
        if( !$test and !$subsample)                          ## if subsampling for the first time...
        { `Utils/xldsubsample $bpedfile $datfile $mapfile $measure $thresh $het $nsubs $seed $nskip`; }
        system("cp Results/Subsample/Subs/sub-* Results/Estimate/");
    
        open(IN, "Results/Subsample/sumstats.out");
        @SUMSTATS = <IN>;
        close(IN);
      }
      else
      {
        print " Processing bootstrap resample $i \r";
        $|++;
        `Utils/xldsubsample $bpedfile $datfile $mapfile $measure $thresh $het $nsubs $seed $nskip`;
      }

      @alodargs = @MCE1wbs = ();                  ## the MCE-1 estimates within a bs resample
      open(IN, "Dats/$datfile");
      @subr = <IN>;
      close(IN);
      for( $k=1; $k<=$nsubs; $k++)
      {
        if( !$tola )
        {
          @lelod = `Utils/xmerlin -p Peds/$bpedfile -d Results/Subsample/Subs/sub-$k -m Maps/$mapfile --npl --info`;
          @LELOD = getnplod( @lelod );
        }
        if( $tola )
        {
          @lelod = `Utils/xmerlin -p Peds/$bpedfile -d Results/Subsample/Subs/sub-$k -m Maps/$mapfile --model Mods/$modfile --info`;
          @LELOD = getplod( @lelod );
        }
        if( $i == 0 ){ open(OUT, ">Results/Estimate/Unaligned-Lods/lelod-$k"); }
        else { open(OUT, ">Results/Subsample/Subs/lelod-$k"); }
        print OUT @LELOD;
        close(OUT);
        push( @LELOD, "2");
        push( @LELOD, "1");
        @fstats = procfx( @LELOD );               ## get mle for a given (subsample, bs resample) pair 
        push( @MCE1wbs, $fstats[0] );             ## store it; wbs --> within a bootstrap resample
        if( $i == 0 ){ push( @alodargs, "Results/Estimate/Unaligned-Lods/lelod-$k"); }
        else{ push( @alodargs, "Results/Subsample/Subs/lelod-$k"); }
      }

      foreach $k (@SUMSTATS)                      ## get mce-1 from dist base srs
      {
        @words = split(" ", $k);
        if( $words[0] eq "Average" ){ $avgnsnps = $words[7]; }
      }
      $rate = $avgnsnps / (scalar(@subr)-2);
 
      for( $k=0; $k<scalar(@subr); $k++)
      {
        @words = split(" ", $subr[$k]);
        $u = rand(1);
        if( $words[0] eq "M" and $u > $rate){ $subr[$k] = "S2 " . $words[1] . "\n"; }
      }
      if( $i == 0 ){ open( OUT, ">Results/Estimate/sub-r"); }
      else { open( OUT, ">Results/Subsample/Subs/sub-r"); }
      print OUT @subr;
      close( OUT );

      if( $i == 0)
      {
        if( !$tola )
        { 
          @lelod = `Utils/xmerlin -p Peds/$bpedfile -d Results/Estimate/sub-r -m Maps/$mapfile --npl --info`; 
          @LELOD = getnplod( @lelod );
        }
        if( $tola )
        {
          @lelod = `Utils/xmerlin -p Peds/$bpedfile -d Results/Estimate/sub-r -m Maps/$mapfile --model Mods/$modfile --info`; 
          @LELOD = getplod( @lelod );
        }
      }
      if( $i > 0 )
      { 
        if( !$tola )
        {
          @lelod = `Utils/xmerlin -p Peds/$bpedfile -d Results/Subsample/Subs/sub-r -m Maps/$mapfile --npl --info`; 
          @LELOD = getnplod( @lelod );
        }
        if( $tola )
        {
          @lelod = `Utils/xmerlin -p Peds/$bpedfile -d Results/Subsample/Subs/sub-r -m Maps/$mapfile --model Mods/$modfile --info`; 
          @LELOD = getplod( @lelod );
        }
      }
      if( $i == 0 ){ open(OUT, ">Results/Estimate/Una*/lelod-r"); }
      else{ open(OUT, ">Results/Subsample/Subs/lelod-r"); }
      print OUT @LELOD;
      close(OUT);
      push( @LELOD, "2");
      push( @LELOD, "1");
      @fstats = procfx( @LELOD );         
      push( @MCE1abs, $fstats[0]);                ## store the mles from a single random (ss, bs) pair
                                                  ## abs --> across bootstrap resamples
                                                  ## wbs --> within bootstrap resamples
      push( @MCE1wbs, "1");
      push( @MCE1wbs, "1");
      @fstats = procfx( @MCE1wbs );               ## get mce-k := mean of @MCE1wbs for a given bs resample 
      push( @MCEk, $fstats[3]);                   ## store it
 
      push( @alodargs, -1 );                      ## -1 -> ESTIMATE; 0 -> ALOD; 1 -> META-LOD; 2 -> CUSTOM
      system("rm -f Results/Subsam*/Subs/sub-*"); ## remove unwanted files in Subsample/Subs directory
      push( @MAE, alod( @alodargs ));             ## get mae for a given bs resample and store it
      
      $seed = int(rand(10000)) + 1;               ## draw a new random seed between [1 - 10,000]
    }
  
    $mae = $MAE[0];                               ## point estimates for location
    $mce = sprintf("%6.4f", $MCEk[0]);
    $srs = sprintf("%6.4f",$MCE1abs[0]);

    $sumxmae  =  $sumxmce  =  $sumxsrs  = 0;      ## compute the CIs
    $sumxxmae =  $sumxxmce =  $sumxxsrs = 0;
    $counter  = 0;
    for( $i=1; $i <= $nboots; $i++)    
    {
      $stable = $mae + $mce + $srs;
      if( 0 < $stable and $stable < 750 )         ## if numerically stable
      {
        $counter++;
        $sumxmae  += $MAE[$i];            
        $sumxmce  += $MCEk[$i];            
        $sumxsrs  += $MCE1abs[$i];
        
        $sumxxmae += ($MAE[$i] * $MAE[$i]);
        $sumxxmce += ($MCEk[$i] * $MCEk[$i]);
        $sumxxsrs += ($MCE1abs[$i] * $MCE1abs[$i]);
      }
    }
    $i = $counter;

    $sdmae = sqrt( $sumxxmae/($i-1) - ($sumxmae * $sumxmae)/($i*($i-1)) );
    $sdmce = sqrt( $sumxxmce/($i-1) - ($sumxmce * $sumxmce)/($i*($i-1)) );
    $sdsrs = sqrt( $sumxxsrs/($i-1) - ($sumxsrs * $sumxsrs)/($i*($i-1)) );
    
    $mael = -1.96 * $sdmae + $mae;  $mael = sprintf("%6.1f", $mael);
    $mcel = -1.96 * $sdmce + $mce;  $mcel = sprintf("%6.1f", $mcel);
    $srsl = -1.96 * $sdsrs + $srs;  $srsl = sprintf("%6.1f", $srsl);
    
    $maeu =  1.96 * $sdmae + $mae;  $maeu = sprintf("%6.1f", $maeu);
    $mceu =  1.96 * $sdmce + $mce;  $mceu = sprintf("%6.1f", $mceu);
    $srsu =  1.96 * $sdsrs + $srs;  $srsu = sprintf("%6.1f", $srsu);
   
    $i = sprintf("%6.1f", 0); 
    if( $mael < 0 ){ $mael = $i; }; $mae = sprintf("%6.1f", $mae);
    if( $mcel < 0 ){ $mcel = $i; }; $mce = sprintf("%6.1f", $mce);
    if( $srsl < 0 ){ $srsl = $i; }; $srs = sprintf("%6.1f", $srs);

    $maelen = sprintf("%6.1f", $maeu - $mael);
    $mcelen = sprintf("%6.1f", $mceu - $mcel);
    $srslen = sprintf("%6.1f", $srsu - $srsl);

    open(OUT,">Results/Estimate/Localize/cis");
    print OUT "\n\n";
    print OUT "    Methods        Estimates   CI Lengths   95% CIs for Trait \n";
    print OUT " -------------------------------------------------------------\n";
    print OUT " Monte Carlo       $mce       $mcelen      ( $mcel, $mceu ) \n";
    print OUT " ALOD Maximizer    $mae       $maelen      ( $mael, $maeu ) \n";
    print OUT " Single Subsample  $srs       $srslen      ( $srsl, $srsu ) \n";
    print OUT "\n\n";
    close( OUT );
    system("cat Results/Estimate/Localize/cis");
    if( !$subsample ){ system("rm -f Results/Subsample/Subs/*");}
    else
    { 
      system("cp Results/Estimate/sub-* Results/Subsample/Subs/"); 
      system("rm -f Results/Subsample/Subs/lelod-*");
    }
  }
}
  
if( $ldinfo ) ## IF LD STATISTICS NEED TO BE REPORTED
{
  if( $ldmeasure eq "rsq") 
  { print " Pairwise Estimates of LD ($ldmeasure) are in Results/LDinfo/pairwise"; }
  else { print " Pairwise Estimates of LD |$ldmeasure| are in Results/LDinfo/pairwise"; }
  @info = `Utils/xldinfo $pedfile $ldmeasure $posld $seed`;
  open(OUT, ">Results/LDinfo/pairwise");
  print OUT @info;
  close(OUT);
  print "\n\n";
}
                        ## IF AVERAGING LODS FROM CONDITIONALLY INDEP SUBSAMPLES,    
if( $meta or $custom )  ## OR IF TOTALING LODS FROM INDEP STUDIES 
{ 
  system("rm -f Results/Meta/*");
  if( $meta )
  { 
     $lodflg = 1;
     $analname = "Meta-LOD";
     print "Computing the Meta-LOD \n"; 
  }
  if( $custom )
  { 
    system("rm -f Results/Custom/*");
    $lodflg = 2;
    $analname = "Custom";
    print "Computing the ALOD from a custom linkage program \n"; 
  }
  @indlods = `ls Lods/`;
  $nlods = $nsubs;
  $nsubs = scalar(@indlods);
  @Indlods = ();
  foreach $k (@indlods)
  { 
    chomp($k);
    $k = "Lods/$k" . " ";
    push( @Indlods, $k); 
  }
  @alodargs = @Indlods;
  push( @alodargs, $lodflg );         ## -1 -> ESTIMATE; 0 -> ALOD; 1 -> META-LOD; 2 -> CUSTOM
  push( @alodargs, $uniqid );
  $META = alod( @alodargs ); 
  print "\n";
  print " ========   ========\n";
  print " Position   $analname\n";
  print " ========   ========\n\n";
  system("paste -d ' ' Results/Meta/cgrid Results/Meta/meta-lod"); 
  print "\n";
  system("cat Results/Meta/mme"); 
  print "\n\n";
  if( $custom )
  {
    # system("mv Results/Test/clods Results/Custom");
    system("mv Results/Meta/cgrid Results/Custom/");
    system("mv Results/Meta/meta-lod Results/Custom/custom-lod");
    # system("mv Results/Meta/mme Results/Custom/mce");
  }
  $nsubs = $nlods;                  ## reset nsubs since it has global scope
}
print "\n THANK YOU for using $version \n\n";

## ================
## END main program
## ================
